// Global Header file of the CForth LowPower project ************************
//
// Author and (c)2020-24 Wolfgang Schemmert. The copyright claim is restricted
// to this file and the depending files "main.c", "main.h", "cforthLP.c" and
// "cforthLP.h"
// Other depending material is published, copyrighted and licensed
// by STMicroelectronics and Segger GmbH
//
// Contact: <www.midi-and-more.de>
// A detailled hardware construction and operation manual
// is provided at this website
//
// ****************************************************************************
//      This program is free software: you can redistribute it and/or modify
//      it under the terms of the GNU General Public License as published by
//      the Free Software Foundation, either version 3 of the License, or
//      (at your option) any later version.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU General Public License
//      along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//      Licence: GPL-3
//
// ****************************************************************************

#ifndef GLOBALDEFINES
#define GLOBALDEFINES

#define REVNUM 4            //describes firmware revision
							
#define HARDWARE_VERSION 1  //describes project specific hardware version
							//only version LQFP32

//for LowPwr it makes sense only in very special cases:
#define MOTOR		//ifndef: PWM2 available

// *******************************

//attributes of KernelOperators
//attribute entries are merged with some other functional flags !!
#define A_IME		0x10000	//ATTRIBUTE_IMMEDIATE, cannot be compiled
#define A_CPO		0x20000	//ATTRIBUTE_COMPILEONLY,refuse work interpreting
#define A_BAK		0x40000	//may be parsed in background at executing state
#define A_CI		(A_CPO | A_IME) //COMPILEONLY IMMEDIATE
							//performs complex operations during compilation
#define A_IB		(A_IME | A_BAK)
#define A_RUN		0x80000 //ATTRIBUTE_RUNTIME,
					//compiled by other operators for special runtime behaviour
					//no direct user access, essential for viewing SEE and STEP
//only attribute of UserToken
#define A_VARCON	0x0C000000

#define ERR_STRINGTOKEN 1
#define ERR_SYNTAX		2
#define ERR_STACK		4
#define ERR_COMPONLY	8
#define ERR_SYSTEM		0x10
#define ERR_USERCODE	0x20
#define ERR_CONTEXT 	0x80
#define ERR_USERCONTEXT	0xA0	//combination of both, save code memory
#define ERR_USERSTRING	0x21	//combination of both, save code memory
#define ERR_CANCEL		0x100
#define ERR_RUNTOKEN	0x200
#define ERR_NUMVAL		0x400
#define ERR_DIVZERO		0x800

#define PAGE205  (0x08006680)	//globals: only gBaud gfNumBase and gProject
#define PAGE206  (0x08006700)	//Project1:single word parameters
#define PAGE207  (0x08006780)	//Project2:single word parameters
#define PAGE208  (0x08006800)	//Project1:gfUserToken (headers)	
#define PAGE220  (0x08006E00)	//Project1:gfTCode
#define PAGE232  (0x08007400)	//Project2:gfUserToken (headers)
#define PAGE244  (0x08007A00)	//Project2:gfTCode
								//includes page255
#define EEP_BASE (0x08080000)

#define KERNELSIZE		128

#define MAXUSER			96//*4 words p.struct=384 words=1536 bytes== 12pages
#define MAXCODE			0x600	//=1536 bytes = 12 pages of each 128 bytes

#define MAXDATASTACK	28
#define MAXRETSTACK		28

#define CODETOP         MAXCODE - 1
#define USERTOP         (uint32_t)&gfUserHeader[MAXUSER]

#define USART_RXBUFLEN 512

#define DEFAULT_BAUD	57	//expanded to 57600, 
							//enter first 2 places, e.g. 96 for 9600
#define DEFAULT_NUMBASE	10 //Decimal, enter 16 for HEX

#define CTRL_B  2
#define CTRL_D  4
#define BKSP  8
#define TAB   9
#define LFF  10		//for Macintosh OS: simply exchange values of CRR and LFF
#define CRR  13		//then New_Line will be interpreted correctly
#define CTRL_R  18
#define ESC  27
#define SPACE  32
#define BACKSLASH 92

#endif  //GLOBALDEFINES

// ##########################################################################
// Version changes:
//REVNUM 1: 19 November2023 copied from CForthLoPofor STM32F042,REVNUM9.
//24Nov23	MSI clock 1.048MHZ introduced as SystemClock. Baudrate fixed:57600
//25Nov23	first Forth code compiled and executed
//			most other CForth features work well in EMPTY project
//			virgin Flash reads 0x00000000, Flash pages are only 128 bytes
//			so a lot of Flash handling has to be reprogrammed
//29NOV23	Flash reorganisation finished
//30NOV23	PB0 is default LED output
//19DEC23	op_MEM now sends block of 128bytes
//			MEM attributes changed to A_BAK
//			bugfix op_DOUNTIL. op_DOWHILE, op_DOLOOP
//20Dec23	bugfix compileTCode: protection against CODETOP overflow
//21Dec23	CheckValidIO improved
//23Dec23   op_QUEST(): printout of CPUclock improved
//			checkProject() introduced, due to overflowing Flash code
//released 25Dec23
//REVNUM->2
//30Dec23	bugfix compilation user header with kernel name (op_colon)
//31Dec23	new gfBackProcess=2: return to higher thread level with CTRL_R
//01Jan24	bugfix gfUserHeader[4] renamed to NSTP (confusion with op_MS)
//			op_MEM now sends block of 128bytes
//			lack of memory: reorg Flash pages:ARRAY not saved anymore in Flas
//			improvement countdown NSTP if stepper is not rotating
//			SLEEP is awaked by Hi>Lo pulse at PA3, PA15, PA10(USART RX)
//02Jan24	pin assignment of PWM1 and PWM2 changed: PWM1=PB1, PWM2=PA0 now
//04Jan24	validIO check and pinI/O vs.PWM,MOT check improvedd
//released 09Jan2024
//REVNUM-->3
//11Jan24	send0X32() modified: always with leading 0x even at small numbers
//18Jan24	bugfix op_SEE()
//			bugfix op_XNOP
//19Jan24	decimal numbers can be entered with leading '%' at any muber base
//			see interpretNumber()
//06March24	ADC multiple read 4x average
//06March24	ADC multiple read 4x average
//09March24	reorg kernel op SAVE and LOAD more Forth typically
//released 14March2024
//REVNUM-->4
//31March24	bugfix cforth.h: DZ and HX have IMMEDIATE flag now
//01Apr24	kernelOp -ROT removed, REGW and REGR implemented new
//02Apr24	doVALUE renamed doVAL, doLIT8B renamed doLIT8

// *******************************